import { Injectable } from '@angular/core';
import { ErrorDialogComponent } from './error-dialog/error-dialog.component';
import { ErrorDialogData } from './error-dialog/_models/error-dialog-data.model';
import { DialogResult } from '../../models/dialog/dialog-result.model';
import { DialogButton } from '../../enums/dialog-button.enum';
import { IErrorDialogService } from './_interfaces/error-dialog-service.interface';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

/**
 *  A service to manage and display error dialogs
 */
@Injectable({
  providedIn: 'root'
})
export class ErrorDialogService implements IErrorDialogService {
  constructor(public dialog: NgbModal) { }

  /**
   * Open an error dialog
   * @param title - The title of the error dialog
   * @param message - The message to display to the user
   */
  public async openErrorDialog(
    title: string,
    message: string
  ): Promise<DialogResult<null>> {
    const dialogData = new ErrorDialogData(title, message);

    const dialogRef = this.dialog.open(ErrorDialogComponent);
    dialogRef.componentInstance.data = dialogData;
    
    return new DialogResult(DialogButton.OK);
  }
}
