import { NgModule, ModuleWithProviders } from '@angular/core';
import { BaseComponent } from './components/base-component/base.component';
import { SafeHtmlPipe } from './util/pipes/safe-html.pipe';
import { ILeathermanAppConfig } from './config/app.config';
import { LeathermanAppConfigInjectionToken } from './config/injection-tokens/leatherman-app-config.injection-token';
import { ErrorDialogServiceInjectionToken } from './config/injection-tokens/error-dialog-service.injection-token';
import { ConfirmDialogServiceInjectionToken } from './config/injection-tokens/confirm-dialog-service.injection-token';
import { AngularModule } from './angular.module';
import { ErrorDialogComponent } from './dialogs/error/error-dialog/error-dialog.component';
import { ConfirmDialogComponent } from './dialogs/confirm/confirm-dialog/confirm-dialog.component';

@NgModule({
  imports: [AngularModule],
  declarations: [
    SafeHtmlPipe,
    BaseComponent,
    ErrorDialogComponent,
    ConfirmDialogComponent,
  ],
  exports: [
    SafeHtmlPipe,
    BaseComponent,
    ErrorDialogComponent,
    ConfirmDialogComponent,
  ]
})

/**
 * A module declaring and exporting Leatherman components
 */
export class LeathermanModule {
  static forRoot(
    appConfig: ILeathermanAppConfig,
    errorDialogService: any,
    confirmDialogService: any
  ): ModuleWithProviders<LeathermanModule> {
    return {
      ngModule: LeathermanModule,
      providers: [
        {
          provide: LeathermanAppConfigInjectionToken,
          useValue: appConfig
        },
        {
          provide: ErrorDialogServiceInjectionToken,
          useClass: errorDialogService
        },
        {
          provide: ConfirmDialogServiceInjectionToken,
          useClass: confirmDialogService
        }
      ]
    };
  }
}
