import { Injectable } from '@angular/core';
import { ComponentCanDeactivate } from './component-can-deactivate';
import { CanDeactivate } from '@angular/router';

/**
 * A service to control whether components can be deactivated
 */
@Injectable()
export class CanDeactivateGuard
  implements CanDeactivate<ComponentCanDeactivate> {
  /**
   * Determine if a component can be deactivated
   * @param component - The component trying to deactivate
   * @returns True if the component can deactivate
   */
  public canDeactivate(component: ComponentCanDeactivate): boolean {
    if (!component.canDeactivate()) {
      if (
        confirm(
          'You have unsaved changes! If you leave, your changes will be lost.'
        )
      ) {
        return true;
      } else {
        return false;
      }
    }
    return true;
  }
}
