import { HostListener, Directive } from '@angular/core';

/**
 * Abstract class to implement can deactivate guard. Components should extend this class and implement CanDeactivate.
 */
@Directive()
export abstract class ComponentCanDeactivate {
  /**
   * Abstract class to determine whether a component can be deactivated
   * @returns True if the component can be deactivated
   */
  abstract canDeactivate(): boolean;

  /**
   * Listen to beforeunload window events and determine whether the current page can be deactivated
   * @param $event - The unload notification event
   */
  @HostListener('window:beforeunload', ['$event'])
  public async unloadNotification($event: any) {
    if ((await this.canDeactivate()) === false) {
      $event.returnValue = true;
    }
  }
}
